﻿using HSR.CloudTaskParallelism.Client.Runtime;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace HSR.CloudTaskParallelism._Samples.Factorize {
  public class Factorization {
    public static void SequentialCompute(long[] inputs) {
      foreach (long number in inputs) {
        Console.WriteLine(_Factorize(number));
      }
    }

    #region Explicit Tasks
    public static void ParallelComputeTasks(long[] inputs) {
      var taskList = new List<Task<long>>();
      foreach (long number in inputs) {
        var task = Task.Factory.StartNew(() => _Factorize(number));
        taskList.Add(task);
      }
      foreach (var task in taskList) {
        Console.WriteLine(task.Result);
      }
    }

    public static void DistributedComputeTasks(long[] inputs) {
      var distribution = new Distribution(AppSettings.Default.TaskExecutionService, AppSettings.Default.TaskExecutionAuthorizationToken);
      var taskList = new List<DistributedTask<long>>();
      foreach (long number in inputs) {
        var task = DistributedTask.New(() => _Factorize(number));
        taskList.Add(task);
      }
      distribution.Start(taskList);
      foreach (var task in taskList) {
        Console.WriteLine(task.Result);
      }
    }
    #endregion

    #region Parallel Loops
    public static void ParallelComputeForLoop(long[] inputs) {
      long[] outputs = new long[inputs.Length];
      Parallel.For(0, inputs.Length, (i) => {
        outputs[i] = _Factorize(inputs[i]);
      });
    }

    public static void DistributedComputeForLoop(long[] inputs) {
      var distribution = new Distribution(AppSettings.Default.TaskExecutionService, AppSettings.Default.TaskExecutionAuthorizationToken);
      long[] outputs = new long[inputs.Length];
      distribution.ParallelFor(0, inputs.Length, (i) => {
        outputs[i] = _Factorize(inputs[i]);
      });
    }
    #endregion

    private static long _Factorize(long number) {
      for (long k = 2; k <= Math.Sqrt(number); k++) {
        if (number % k == 0) { return k; }
      }
      return number; // not factorizable
    }
  }
}
