﻿using System;

namespace HSR.CloudTaskParallelism._Samples.Factorize {
  // Series of test input numbers for factorization
  public class InputGenerator {
    private readonly Random random ;

    public InputGenerator(int seed) {
      random = new Random(seed);
    }

    // Sequence of randomly selected hard-to-factorize numbers
    public long[] GetInputSeries(int amount) {
      long[] numbers = new long[amount];
      for (int i = 0; i < amount; i++) {
        numbers[i] = _GetRandomPrime() * _GetRandomPrime();
      }
      return numbers;
    }

    private long _GetPositiveRandom() {
      return Math.Abs(random.Next() * random.Next()) / 2;
    }

    private long _GetRandomPrime() {
      return _NextPrint(_GetPositiveRandom());
    }

    private long _NextPrint(long number) {
      while (!_IsPrime(number)) { number++; }
      return number;
    }

    private bool _IsPrime(long number) {
      for (long k = 2; k <= Math.Sqrt(number); k++) {
        if (number % k == 0) { return false; }
      }
      return true;
    }
  }
}
