﻿using HSR.CloudTaskParallelism.Client.Runtime;
using System;
using System.Threading.Tasks;
using System.Windows;

namespace HSR.CloudTaskParallelism._Samples.KnightTours {
  public partial class MainWindow : Window {
    private Distribution _distribution = new Distribution(AppSettings.Default.TaskExecutionServiceUri, AppSettings.Default.TaskExecutionAuthorizationToken);

    public MainWindow() {
      InitializeComponent();
    }

    private async void computeButton_Click(object sender, RoutedEventArgs e) {
      IsEnabled = false;
      resultTextBlock.Text = "Results: ";
      int nofRows = int.Parse(rowsTextBox.Text);
      int nofCols = int.Parse(colsTextBox.Text);

      KnightTour knightTour = new KnightTour(nofRows, nofCols);

      await _Measure("Distributed", () => knightTour.DistributedCountKnightTours(_distribution));
      await _Measure("Parallel", () => knightTour.ParallelCountKnightTours());
      await _Measure("Sequential", () => knightTour.SequentialCountKnightTours());
      IsEnabled = true;
    }

    private async Task _Measure(string name, Func<long> function) {
      double totalTime = 0;
      long result = 0;
      await Task.Run(() => {
        DateTime startTime = DateTime.Now;
        result = function();
        DateTime stopTime = DateTime.Now;
        totalTime = (stopTime - startTime).TotalMilliseconds;
      });
      resultTextBlock.Text = resultTextBlock.Text + Environment.NewLine +
        string.Format("{0} ({1} tours) {2} ms", name, result, totalTime);
    }
  }
}
