﻿using HSR.CloudTaskParallelism.Client.Runtime;
using System.Threading.Tasks;

namespace HSR.CloudTaskParallelism._Samples.Mandelbrot {
  public class Mandelbrot {
    private const int _MaxIterations = 512;

    public static byte[][] SequentialCompute(double left, double top, double width, double height, int horizontalPixels, int verticalPixels) {
      byte[][] pixels = new byte[verticalPixels][];
      for (int row = 0; row < verticalPixels; row++) {
        _MandelbrotRow(left, top, width, height, horizontalPixels, verticalPixels, pixels, row);
      }
      return pixels;
    }

    public static byte[][] ParallelCompute(double left, double top, double width, double height, int horizontalPixels, int verticalPixels) {
      byte[][] pixels = new byte[verticalPixels][];
      Parallel.For(0, verticalPixels, (row) => {
        _MandelbrotRow(left, top, width, height, horizontalPixels, verticalPixels, pixels, row);
      });
      return pixels;
    }

   public static byte[][] DistributedCompute(Distribution distribution, double left, double top, double width, double height, int horizontalPixels, int verticalPixels) {
      byte[][] pixels = new byte[verticalPixels][];
      distribution.ParallelFor(0, verticalPixels, (row) => {
        _MandelbrotRow(left, top, width, height, horizontalPixels, verticalPixels, pixels, row);
      });
      return pixels;
    }

   private static void _MandelbrotRow(double left, double top, double width, double height, int horizontalPixels, int verticalPixels, byte[][] pixels, int row) {
     double y0 = row * height / verticalPixels + top;
     pixels[row] = new byte[horizontalPixels];
     for (int col = 0; col < horizontalPixels; col++) {
       Complex c = new Complex(col * width / horizontalPixels + left, y0);
       Complex z = c;
       int iteration = 0;
       while (iteration < _MaxIterations && z.Magnitude <= 4) {
         z = (z * z) + c;
         iteration++;
       }
       pixels[row][col] = (byte)iteration;
     }
   }
  }
}
