﻿using HSR.CloudTaskParallelism.Client.Runtime;
using System;
using System.Threading.Tasks;
using System.Windows;

namespace HSR.CloudTaskParallelism._Samples.Primes {
  public partial class MainWindow : Window {
    private Distribution _distribution = new Distribution(AppSettings.Default.TaskExecutionServiceUri, AppSettings.Default.TaskExecutionAuthorizationToken);
    
    public MainWindow() {
      InitializeComponent();
    }

    private async void computeButton_Click(object sender, RoutedEventArgs e) {
      IsEnabled = false;
      resultTextBlock.Text = "Results: ";
      long lowerLimit = Math.Max(long.Parse(lowerTextBox.Text), 2);
      long upperLimit = Math.Max(long.Parse(upperTextBox.Text), 2);

      await _Measure("Distributed", () => PrimesScanner.DistributedCountPrimes(_distribution, lowerLimit, upperLimit));
      await _Measure("Parallel", () => PrimesScanner.ParallelCountPrimes(lowerLimit, upperLimit));
      await _Measure("Sequential", () => PrimesScanner.SequentialCountPrimes(lowerLimit, upperLimit));
      IsEnabled = true;
    }

    private async Task _Measure(string name, Func<long> function) {
      double totalTime = 0;
      long result = 0;
      await Task.Run(() => {
        DateTime startTime = DateTime.Now;
        result = function();
        DateTime stopTime = DateTime.Now;
        totalTime = (stopTime - startTime).TotalMilliseconds;
      });
      resultTextBlock.Text = resultTextBlock.Text + Environment.NewLine + 
        string.Format("{0} ({1} primes) {2} ms", name, result, totalTime);
    }
  }
}
