﻿using HSR.CloudTaskParallelism.Client.Runtime;
using System;
using System.Collections.Concurrent;
using System.Threading;
using System.Threading.Tasks;

namespace HSR.CloudTaskParallelism._Samples.Primes {
  // brute force (could alternatively use sieving - starting however from 2)
  public static class PrimesScanner {
    public static long SequentialCountPrimes(long fromInclusive, long toExclusive) {
      long count = 0;
      for (long number = fromInclusive; number < toExclusive; number++) {
        if (_IsPrime(number)) {
          count++;
        }
      }
      return count;
    }

    public static long ParallelCountPrimes(long fromInclusive, long toExclusive) {
      long count = 0;
      // partitioner to avoid huge amount of very small tasks
      Parallel.ForEach(Partitioner.Create(fromInclusive, toExclusive), (range) => {
        for (long number = range.Item1; number < range.Item2; number++) {
          if (_IsPrime(number)) {
            Interlocked.Increment(ref count); // only works on local SMP.
          }
        }
      });
      return count;
    }

    public static long DistributedCountPrimes(Distribution distribution, long fromInclusive, long toExclusive) {
      long fullRange = Math.Max(0, toExclusive - fromInclusive);
      long nofPartitions = Math.Min(256, fullRange);
      long partitionSize = (fullRange + nofPartitions - 1) / nofPartitions;
      long count = 0;
      distribution.ParallelFor(0, nofPartitions, (partition) => {
        long innerLower = fromInclusive + partition * partitionSize;
        long innerUpper = Math.Min(toExclusive, innerLower + partitionSize);
        int innerCount = 0;
        for (long number = innerLower; number < innerUpper; number++) {
          if (_IsPrime(number)) {
            innerCount++;
          }
        }
        return innerCount;
      }, (_, innerCount) => {
        count += innerCount;
      });
      return count;
    }
    
    private static bool _IsPrime(long number) {
      for (long k = 2; k <= Math.Sqrt(number); k++) {
        if (number % k == 0) { return false; }
      }
      return true;
    }
  }
}
